/*
          LTR11.
            ,    
           :
    ltr11_recv  slot crate_serial srvip
    :
        slot         -   ( 1  16)
        crate_serial -    (   )
        srvip        - ip-  ltrd  LtrServer (     )
     :    ,       

        4-     400 .
             .

         ,   
      .

           Windows   CTRL+C  Linux.

      VisualStudio:
          Visual Studio,     
    ( (Project) ->  (Properties) ->   (Configuration Properties)
    -> /++ ->  (General) ->    (Additional Include Directories))
            (ltr11api.h    
         ltr/include   )
        .lib   <  >/lib/msvc
    ( (Project) ->  (Properties) ->   (Configuration Properties) ->
     (Linker) ->  (General) ->    (Additional Library Directories)).

    !:      Visual Studio    
       ,    :
      (File) ->    (Advanced Save Options)...
        (Encoding)   (UTF8,  )/Unicode (UTF-8 with signature)
        .    ,   
           ( Consolas).
*/


#include "ltr/include/ltr11api.h"
/*    */
#include <stdio.h>
#include <stdlib.h>
#ifdef _WIN32
#include <locale.h>
#include <conio.h>
#else
#include <signal.h>
#include <unistd.h>
#endif


/*    ,    */
#define RECV_BLOCK_CH_SIZE  4096*8
/*       (   ) */
#define RECV_TOUT  4000


typedef struct {
    int slot;
    const char *serial;
    DWORD addr;
} t_open_param;


/*      */
static int f_out = 0;

#ifndef _WIN32
/*     Linux */
static void f_abort_handler(int sig) {
    f_out = 1;
}
#endif


/*    .   ,  
 *   :
 * 1  -   ( 1  16)
 * 2  -   
 * 3  - ip-  */
static int f_get_params(int argc, char** argv, t_open_param* par) {
    int err = 0;
    par->slot = LTR_CC_CHNUM_MODULE1;
    par->serial = "";
    par->addr = LTRD_ADDR_DEFAULT;


    if (argc > 1)
        par->slot = atoi(argv[1]);
    if (argc > 2)
        par->serial = argv[2];
    if (argc > 3) {
        int a[4],i;
        if (sscanf(argv[3], "%d.%d.%d.%d", &a[0], &a[1], &a[2], &a[3])!=4) {
            fprintf(stderr, "  IP-!!\n");
            err = -1;
        }

        for (i=0; (i < 4) && !err; i++) {
            if ((a[i]<0) || (a[i] > 255)) {
                fprintf(stderr, " IP-!!\n");
                err = -1;
            }
        }

        if (!err) {
            par->addr = (a[0] << 24) | (a[1]<<16) | (a[2]<<8) | a[3];
        }
    }
    return err;
}



/*-----------------------------------------------------------------------------------------------*/
int main(int argc, char* argv[]) {
    TLTR11 hltr11;
    INT  err;
    t_open_param par;
#ifndef _WIN32
    struct sigaction sa;
    /*   Linux      ,
           */
    sa.sa_handler = f_abort_handler;
    sigaction(SIGTERM, &sa, NULL);
    sigaction(SIGINT, &sa, NULL);
    sigaction(SIGABRT, &sa, NULL);
#endif
#ifdef _WIN32
    /*         Windows*/
    setlocale(LC_ALL, "");
#endif
    err = f_get_params(argc, argv, &par);

    if (!err) {
        /*    */
        LTR11_Init(&hltr11);
        /*     ,     */
        err = LTR11_Open(&hltr11, par.addr, LTRD_PORT_DEFAULT, par.serial, par.slot);
    }
    if (err != LTR_OK) {
        fprintf(stderr, "     .  %d (%s)\n",
                err, LTR11_GetErrorString(err));
    } else {
        /*      flash- */
        err = LTR11_GetConfig(&hltr11);
        if (err != LTR_OK) {
            fprintf(stderr, "     .  %d: %s\n", err,
                    LTR11_GetErrorString(err));
        } else {  /*    */
            /*     */
            printf( "  :\n"
                    "     : %s\n"
                    "      : %s\n"
                    "     : %u.%u\n",
                       hltr11.ModuleInfo.Name, hltr11.ModuleInfo.Serial,
                      (hltr11.ModuleInfo.Ver >> 8) & 0xFF, hltr11.ModuleInfo.Ver & 0xFF);
            fflush(stdout);


            
            /*     */
            /*     -  */
            hltr11.StartADCMode = LTR11_STARTADCMODE_INT;
            /*    -  */
            hltr11.InpMode      = LTR11_INPMODE_INT;
            /*    - 4 */
            hltr11.LChQnt       = 4;
            /*     */
            /*  - 10,  - 16-,   - 1 */
            hltr11.LChTbl[0] = LTR11_CreateLChannel(0, LTR11_CHMODE_DIFF, LTR11_CHRANGE_10000MV);
            /*  - 2.5,  -   ,   - 2 */
            hltr11.LChTbl[1] = LTR11_CreateLChannel(1, LTR11_CHMODE_ZERO, LTR11_CHRANGE_2500MV);
            /*  - 0.6,  - 32- ( 1..16),   - 3 */
            hltr11.LChTbl[2] = LTR11_CreateLChannel(2, LTR11_CHMODE_COMM, LTR11_CHRANGE_625MV);
            /*  - 0.15,  - 32- ( 17..32),   - 25 */
            hltr11.LChTbl[3] = LTR11_CreateLChannel(24, LTR11_CHMODE_COMM, LTR11_CHRANGE_156MV);
            /*    */
            hltr11.ADCMode = LTR11_ADCMODE_ACQ;
            /*   - 400 .    */
            LTR11_FindAdcFreqParams(400000, &hltr11.ADCRate.prescaler,
                                    &hltr11.ADCRate.divider, NULL);
            
            /*     */
            err = LTR11_SetADC(&hltr11);
            if (err != LTR_OK) {
                fprintf(stderr, "    .  %d: %s\n", err,
                        LTR11_GetErrorString(err));
            }
        }

        if (err == LTR_OK) {
            DWORD recvd_blocks=0;
            INT recv_data_cnt = RECV_BLOCK_CH_SIZE * hltr11.LChQnt;
            DWORD  *rbuf = (DWORD*)malloc(recv_data_cnt*sizeof(rbuf[0]));
            double *data = (double *)malloc(recv_data_cnt*sizeof(data[0]));
            if ((rbuf==NULL) || (data==NULL)) {
                fprintf(stderr, "  !\n");
                err = LTR_ERROR_MEMORY_ALLOC;
            } else {
                /*    */
                err = LTR11_Start(&hltr11);
                if (err != LTR_OK) {
                    fprintf(stderr, "    .  %d: %s\n", err,
                            LTR11_GetErrorString(err));
                } else {
                    INT stop_err = 0;
                    printf("  .    %s\n",
#ifdef _WIN32
                           " "
#else
                           "CTRL+C"
#endif
                           );
                    fflush(stdout);

                    while (!f_out && (err==LTR_OK)) {
                        INT recvd;
                        /*       */
                        DWORD tout = RECV_TOUT + (DWORD)(RECV_BLOCK_CH_SIZE/hltr11.ChRate + 1);
                        /*    LTR11 */
                        recvd = LTR11_Recv(&hltr11, rbuf, NULL, recv_data_cnt, tout);
                        /*       */
                        if (recvd<0) {
                            err = recvd;
                            fprintf(stderr, "  .  %d:%s\n",
                                    err, LTR11_GetErrorString(err));
                        } else if (recvd!=recv_data_cnt) {
                            fprintf(stderr, "  .  %d,  %d\n",
                                    recv_data_cnt, recvd);
                            err = LTR_ERROR_RECV_INSUFFICIENT_DATA;
                        } else {
                            /*        */
                            err = LTR11_ProcessData(&hltr11, rbuf, data, &recvd, TRUE, TRUE);
                            if (err!=LTR_OK) {
                                fprintf(stderr, "  .  %d:%s\n",
                                        err, LTR11_GetErrorString(err));
                            } else {
                                INT ch;
                                recvd_blocks++;
                                /*       */
                                printf(" %4d: ", recvd_blocks);

                                for (ch=0; ch < hltr11.LChQnt; ch++) {
                                    printf("%8.2f", data[ch]);

                                    if (ch==(hltr11.LChQnt-1)) {
                                        printf("\n");
                                    } else {
                                        printf(",  ");
                                    }
                                }
                                fflush(stdout);
                            }
                        }
#ifdef _WIN32
                        /*      */
                        if (err==LTR_OK) {
                            if (_kbhit())
                                f_out = 1;
                        }
#endif
                    } //while (!f_out && (err==LTR_OK))


                    /*    */
                    stop_err = LTR11_Stop(&hltr11);
                    if (stop_err!=LTR_OK) {
                        fprintf(stderr, "    .  %d: %s\n", err,
                            LTR11_GetErrorString(stop_err));
                        if (err == LTR_OK)
                            err = stop_err;
                    } else {
                        printf("  !\n");
                    }
                }

                free(rbuf);
                free(data);
            }
        }
    }

    if (LTR11_IsOpened(&hltr11) == LTR_OK) {
        /*      */
        LTR11_Close(&hltr11);
    }
        
    return err;
}


